@@ -1,18 +1,15 @@
-#ifndef SUPPORT_H
-#define SUPPORT_H
+#ifndef _SUPPORT_H_
+#define _SUPPORT_H_
 
 #define InnerWidth(w) (w->Width - w->BorderLeft - w->BorderRight)
 #define InnerHeight(w) (w->Height - w->BorderTop - w->BorderBottom)
 
 /* Some MUI Support Functions */
-
+#if 0
 LONG xget(Object *obj,ULONG attribute);
 char *getstr(Object *obj);
 BOOL getbool(Object *obj);
-Object *MUI_NewObject(char *classname,Tag tag1,...);
-BOOL MUI_AslRequestTags(APTR requester, Tag Tag1, ...);
-Object *MUI_MakeObject(LONG type,...);
-LONG    MUI_Request(APTR app,APTR win,LONGBITS flags,char *title,char *gadgets,char *format,...);
+
 Object *MakeLabel(STRPTR str);
 Object *MakeLabel1(STRPTR str);
 Object *MakeLabel2(STRPTR str);
@@ -28,7 +25,7 @@ Object *MakeLV(APTR pool);
 ULONG DoSuperNew(struct IClass *cl,Object *obj,ULONG tag1,...);
 
 #define nnsetstring(obj,s)  nnset(obj,MUIA_String_Contents,s)
-
+#endif
 
 LONG StrLen( const STRPTR str);
 STRPTR StrCopy( const STRPTR str );
@@ -38,5 +35,7 @@ ULONG GetBestID( ULONG width, ULONG height, ULONG depth );
 STRPTR GetDisplayName(ULONG displayid);
 APTR FindUserData( struct Menu *menu, APTR userdata);
 
+struct Library *OpenLibraryInterface(STRPTR name, int version, void *interface_ptr);
+void CloseLibraryInterface(struct Library *lib, void *interface);
 
-#endif
+#endif /* _SUPPORT_H_ */
