@@ -62,7 +62,7 @@
 #ifdef SOUND
 #include "sound.h"
 #endif
-#if (defined(SOUND) && !defined(DREAMCAST)) || defined(VIDEO_RECORDING)
+#if defined(HAVE_LIBPNG) || defined(HAVE_LIBZ) || defined(AUDIO_RECORDING) || defined(VIDEO_RECORDING)
 #include "file_export.h"
 #endif
 
@@ -180,6 +180,8 @@ int CFG_LoadConfig(const char *alternate_config_filename)
 				Util_strlcpy(Devices_atari_h_dir[3], ptr, FILENAME_MAX);
 			else if (strcmp(string, "HD_READ_ONLY") == 0)
 				Devices_h_read_only = Util_sscandec(ptr);
+			else if (strcmp(string, "HD_DEVICE_NAME") == 0)
+				Devices_h_device_name = *ptr;
 
 			else if (strcmp(string, "PRINT_COMMAND") == 0) {
 				if (!Devices_SetPrintCommand(ptr))
@@ -338,7 +340,7 @@ int CFG_LoadConfig(const char *alternate_config_filename)
 			else if (Sound_ReadConfig(string, ptr)) {
 			}
 #endif /* defined(SOUND) && defined(SOUND_THIN_API) */
-#if (defined(SOUND) && !defined(DREAMCAST)) || defined(VIDEO_RECORDING)
+#if defined(HAVE_LIBPNG) || defined(HAVE_LIBZ) || defined(AUDIO_RECORDING) || defined(VIDEO_RECORDING)
 			else if (File_Export_ReadConfig(string, ptr)) {
 			}
 #endif
@@ -397,6 +399,7 @@ int CFG_WriteConfig(void)
 	for (i = 0; i < 4; i++)
 		fprintf(fp, "H%c_DIR=%s\n", '1' + i, Devices_atari_h_dir[i]);
 	fprintf(fp, "HD_READ_ONLY=%d\n", Devices_h_read_only);
+	fprintf(fp, "HD_DEVICE_NAME=%c\n", Devices_h_device_name);
 
 #ifdef HAVE_SYSTEM
 	fprintf(fp, "PRINT_COMMAND=%s\n", Devices_print_command);
@@ -483,7 +486,7 @@ int CFG_WriteConfig(void)
 #if defined(SOUND) && defined(SOUND_THIN_API)
 	Sound_WriteConfig(fp);
 #endif /* defined(SOUND) && defined(SOUND_THIN_API) */
-#if (defined(SOUND) && !defined(DREAMCAST)) || defined(VIDEO_RECORDING)
+#if defined(HAVE_LIBPNG) || defined(HAVE_LIBZ) || defined(AUDIO_RECORDING) || defined(VIDEO_RECORDING)
 	File_Export_WriteConfig(fp);
 #endif
 #ifdef SUPPORTS_PLATFORM_CONFIGSAVE
