@@ -11,19 +11,21 @@ int File_Export_Initialise(int *argc, char *argv[]);
 int File_Export_ReadConfig(char *string, char *ptr);
 void File_Export_WriteConfig(FILE *fp);
 
+#if defined(SCREENSHOTS) || defined(AUDIO_RECORDING) || defined(VIDEO_RECORDING)
 void fputw(UWORD, FILE *fp);
 void fputl(ULONG, FILE *fp);
+#endif
 
+#if defined(AUDIO_RECORDING) || defined(VIDEO_RECORDING)
 extern char *FILE_EXPORT_error_message;
 void File_Export_SetErrorMessage(const char *string);
 void File_Export_SetErrorMessageArg(const char *format, const char *arg);
 
-#ifdef MULTIMEDIA
 int File_Export_IsRecording(void);
 int File_Export_StopRecording(void);
 int File_Export_StartRecording(const char *fileName);
 
-#ifdef SOUND
+#ifdef AUDIO_RECORDING
 int File_Export_GetNextSoundFile(char *buffer, int bufsize);
 int File_Export_WriteAudio(const UBYTE *samples, int num_samples);
 #endif
@@ -34,9 +36,9 @@ int File_Export_WriteVideo(void);
 #endif
 
 int File_Export_GetRecordingStats(int *seconds, int *size, char **media_type);
-#endif /* MULTIMEDIA */
+#endif /* defined(AUDIO_RECORDING) || defined(VIDEO_RECORDING) */
 
-#if !defined(BASIC) && !defined(CURSES_BASIC)
+#ifdef SCREENSHOTS
 int File_Export_ImageTypeSupported(const char *id);
 int File_Export_SaveScreen(const char *filename, UBYTE *ptr1, UBYTE *ptr2);
 #endif
