@@ -21,6 +21,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 extern int RTIME_enabled;
 
+int RTIME_ReadConfig(char *string, char *ptr);
+void RTIME_WriteConfig(FILE *fp);
 int RTIME_Initialise(int *argc, char *argv[]);
 UBYTE RTIME_GetByte(void);
 void RTIME_PutByte(UBYTE byte);
