@@ -172,8 +172,11 @@ int main(int argc, char **argv)
 	if (!Atari800_Initialise(&argc, argv))
 		return 3;
 
-	if(Atari800_start_in_monitor)
-		PLATFORM_Exit(TRUE);
+	if(Atari800_start_in_monitor) {
+		if (!Atari800_Exit(TRUE))
+			/* if 'quit' typed in monitor, exit emulator */
+			exit(0);
+	}
 
 	/* main loop */
 	for (;;) {
