@@ -29,17 +29,8 @@
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h> /* getcwd */
-#endif
-#ifdef HAVE_DIRECT_H
-#include <direct.h> /* getcwd on MSVC*/
-#endif
 #ifdef HAVE_LIBZ
 #include <zlib.h>
-#else
-#define gzFile char *
-#define Z_OK 0
 #endif
 #ifdef LIBATARI800
 #include "libatari800/init.h"
@@ -80,6 +71,11 @@
 #define SAVE_VERSION_NUMBER 8 /* Last changed after Atari800 3.1.0 */
 
 #if defined(MEMCOMPR) || defined(LIBATARI800)
+/* libatari800 pretends to care about libz but it doesn't */
+#ifdef LIBATARI800
+#define gzFile char *
+#define Z_OK 0
+#endif
 static gzFile mem_open(const char *name, const char *mode);
 static int mem_close(gzFile stream);
 static size_t mem_read(void *buf, size_t len, gzFile stream);
@@ -303,17 +299,15 @@ void StateSav_ReadINT(int *data, int num)
 void StateSav_SaveFNAME(const char *filename)
 {
 	UWORD namelen;
-#ifdef HAVE_GETCWD
 	char dirname[FILENAME_MAX]="";
 
 	/* Check to see if file is in application tree, if so, just save as
 	   relative path....*/
-	if (getcwd(dirname, FILENAME_MAX) != NULL) {
-		if (strncmp(filename, dirname, strlen(dirname)) == 0)
-			/* XXX: check if '/' or '\\' follows dirname in filename? */
-			filename += strlen(dirname) + 1;
+	Util_getcwd(dirname, FILENAME_MAX);
+	if (strncmp(filename, dirname, strlen(dirname)) == 0) {
+		/* XXX: check if '/' or '\\' follows dirname in filename? */
+		filename += strlen(dirname) + 1;
 	}
-#endif
 
 	namelen = strlen(filename);
 	/* Save the length of the filename, followed by the filename */
